@@ -1,10 +1,10 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2010 Chris Liechti <cliechti@gmx.net>
-# All Rights Reserved.
-# Simplified BSD License (see LICENSE.txt for full text)
-
+# This file is part of https://github.com/zsquareplusc/python-msp430-tools
+# (C) 2010 Chris Liechti <cliechti@gmx.net>
+#
+# SPDX-License-Identifier:    BSD-3-Clause
 """\
 This is a parser for configuration files that describe the memory map of a
 micro controller.
@@ -20,63 +20,8 @@ using the same rules described here.
 
 Supported commands are:
 
-segment <name>  <memory_range>
-    Defines a segment.
-    Previously set flags are applied and cleared.
-    <memory_range> can be like "0x0200-0x0300" or a set of "key:value" pairs.
 
-    Supported pairs are: "in:<segmen_name>" "mirror:<segment_name>"
 
-symbol <name> <address>
-    Defines a symbol with the value specified. <address> can also be a computed
-    value. e.g. "in:RAM,location:end".
-    Supported are: "in:<segment_name>" and "location:[start|end]".  These
-    values are computed at load time, i.e. the segment still have the address
-    range specified in the definion (opposed to the values after the linker has
-    "shrinked" the segments to the size of actually present data). Note that
-    "location:end" is the segments last address plus one (end is exclusive in
-    this case)
-
-memory-map-begin
-    Start the definition of a memory map for a MCU. It's expected that the
-    'name' and 'segment' commands are used to define a memory map.
-
-memory-map-end
-    Terminate memory map definition.
-
-name <name>
-    Set the name of a memory map.
-
-based-on <name>
-    Tell that a memory map definition builds on an other definition.
-    All the definitions are merged when used.
-
-template-begin
-    Begin a template. What follows is the text of the template itself.  It may
-    contain special words that will be used as variables.  They can have any
-    name. The template text is finished with the command
-    'template_variables'.
-
-template-variables
-    The names of the variables follow. These are the words that are used in the
-    previously defined template text. This section is terminated by
-    'template_values'.
-
-template-values
-    Values are following until 'template_end' is found. Each word that is read
-    is assigned to the list of values. When the list of values has the same
-    length as the list of variables are they replaced in the template text and
-    the resulting text is parsed again.
-
-template-end
-    Denotes the end of a values section in a template.
-
-read-only
-    Set flag that the next defined segment is read-only (not programmed to
-    target).
-
-programmable
-    Set flag that the next defined segment is programmed on the target.
 
 Words in brackets ("<...>") mean that the words following the command are
 consumed as parameter to the command.
@@ -129,13 +74,23 @@ Example::
     template-end
 """
 
-import rpn
+from . import rpn
 import pkgutil
 
+
 class MCUDefintitionError(Exception):
     """for errors in de MCU definition file"""
 
 
+def filtered_words(words, mapping):
+    """\
+    Go through a sequence of words. Words occuring in mapping are replaced,
+    others are passed on as-is.
+    """
+    for word in words:
+        yield mapping.get(word, word)
+
+
 class MCUDefintitions(rpn.RPN):
 
     def __init__(self):
@@ -155,7 +110,53 @@ class MCUDefintitions(rpn.RPN):
 
     @rpn.word('TEMPLATE-BEGIN')
     def word_TEMPLATE_BEGIN(self, stack):
-        """Read and execute a template."""
+        """\
+        Read and execute a template. This command consists of 3 sections:
+
+        - definition of a text
+        - definition of a set of variables
+        - values for the variables
+
+        template-begin
+            Begin a template. What follows is the text of the template itself.  It may
+            contain special words that will be used as variables.  They can have any
+            name. The template text is finished with the command
+            'template_variables'.
+
+        template-variables
+            The names of the variables follow. These are the words that are used in the
+            previously defined template text. This section is terminated by
+            'template_values'.
+
+        template-values
+            Values are following until 'template_end' is found. Each word that is read
+            is assigned to the list of values. When the list of values has the same
+            length as the list of variables are they replaced in the template text and
+            the resulting text is parsed again.
+
+        template-end
+            Denotes the end of a values section in a template.
+
+        Example::
+
+            template-begin
+                memory-map-begin
+                    name        <MCU>
+                    based-on    MSP430F2xx
+                                 segment     RAM             <RAM>
+                    programmable segment     FLASH           <FLASH>
+                memory-map-end
+            template-variables
+                <MCU>           <RAM>           <FLASH>
+            template-values
+                MSP430F2001     0x0200-0x027f   0xfc00-0xffdf   # 128B RAM, 1kB Flash
+                MSP430F2002     0x0200-0x027f   0xfc00-0xffdf   # 128B RAM, 1kB Flash
+                MSP430F2003     0x0200-0x027f   0xfc00-0xffdf   # 128B RAM, 1kB Flash
+                MSP430F2011     0x0200-0x027f   0xf800-0xffdf   # 128B RAM, 2kB Flash
+                MSP430F2012     0x0200-0x027f   0xf800-0xffdf   # 128B RAM, 2kB Flash
+                MSP430F2013     0x0200-0x027f   0xf800-0xffdf   # 128B RAM, 2kB Flash
+            template-end
+        """
         template = []
         # read the template itself
         while True:
@@ -163,7 +164,7 @@ class MCUDefintitions(rpn.RPN):
             if word.lower() == 'template-variables':
                 break
             template.append(word)
-        template = ' '.join(template)
+        #~ template = ' '.join(template)
         # read the variables
         template_variables = []
         while True:
@@ -177,43 +178,81 @@ class MCUDefintitions(rpn.RPN):
             word = self.next_word()
             if word.lower() == 'template-end':
                 if template_row:
-                    raise MCUDefintitionError('Values in template values section left')
+                    raise MCUDefintitionError('number of values in template not a multiple of number of variables')
                 break
             # collect values
             template_row.append(word)
             # enough values for template -> apply
             if len(template_row) == len(template_variables):
-                t = template
-                for k, v in zip(template_variables, template_row):
-                    t = t.replace(k, v)
-                self.memory_maps.update(parse_words(iter(t.split())))
+                self.memory_maps.update(parse_words(filtered_words(template, dict(zip(template_variables, template_row)))))
                 template_row = []
 
-
     @rpn.word('PROGRAMMABLE')
     def word_PROGRAMMABLE(self, stack):
-        """Set flag for the following SECTION"""
+        """\
+        Set flag that the next defined segment is programmed on the target.
+        Example::
+
+            programmable segment     .text           in:FLASH
+        """
         if self.memory_map is None:
             raise MCUDefintitionError('flags outside memory map definition not allowed')
         self.flags.append('programmable')
 
     @rpn.word('READ-ONLY')
     def word_READ_ONLY(self, stack):
-        """Set flag for the following SECTION"""
+        """\
+        Set flag that the next defined segment is read-only (not programmed to
+        target).
+        Example::
+
+            read-only  segment bootloader 0x0c00-0x0fff
+        """
         if self.memory_map is None:
             raise MCUDefintitionError('flags outside memory map definition not allowed')
         self.flags.append('read-only')
 
     @rpn.word('MEMORY-MAP-BEGIN')
     def word_MEMORY_MAP_BEGIN(self, stack):
-        """Start a new memory map"""
+        """
+        Start the definition of a memory map for a MCU. It's expected that the
+        NAME_ and SEGMENT_ commands are used to define a memory map.
+        Example::
+
+            memory-map-begin
+                name        LOGICAL
+                # declare a "DATA" segment at the beginning of RAM
+                segment     .data           in:RAM
+                segment     .bss            in:RAM
+                segment     .noinit         in:RAM
+                symbol      _stack          in:RAM,location:end
+
+
+                # declare multiple segments that are located in FLASH
+                programmable segment     .text           in:FLASH
+                programmable segment     .const          in:FLASH
+                programmable segment     .data_init      in:FLASH,mirror:.data
+            memory-map-end
+
+            memory-map-begin
+                name         MSP430F2xx
+                based-on     LOGICAL
+                read-only    segment     .bootloader     0x0c00-0x0fff
+                programmable segment     .infomem        0x1000-0x10ff
+                programmable segment     .infoD          0x1000-0x103f
+                programmable segment     .infoC          0x1040-0x107f
+                programmable segment     .infoB          0x1080-0x10bf
+                programmable segment     .infoA          0x10c0-0x10ff
+                programmable segment     .vectors        0xffe0-0xffff
+            memory-map-end
+        """
         if self.memory_map is not None:
             raise MCUDefintitionError('MEMORY-MAP-BEGIN without terminating the last map')
         self.memory_map = {}
 
     @rpn.word('MEMORY-MAP-END')
     def word_MEMORY_MAP_END(self, stack):
-        """Terminate current memory map definition"""
+        """Terminate current memory map definition. See `MEMORY-MAP-BEGIN`_."""
         if '__name__' not in self.memory_map:
             raise MCUDefintitionError('each memory map requires a NAME')
         self.memory_maps[self.memory_map['__name__']] = self.memory_map
@@ -221,7 +260,28 @@ class MCUDefintitions(rpn.RPN):
 
     @rpn.word('SEGMENT')
     def word_SEGMENT(self, stack):
-        """Define a segment. Consume flags that have been set"""
+        """\
+        Example::
+
+            segment <name>  <memory_range>
+
+        Defines a segment.
+        Previously set flags are applied and cleared.
+        ``<memory_range>`` can be an address range like ``0x0200-0x0300`` or a
+        set of ``key:value`` pairs:
+
+        ``in:<segment_name>``
+            This segment is placed within an other parent segment. The memory
+            range is inherited from the parent. Multiple segments can be placed
+            in one parent segment.
+
+        ``mirror:<segment_name>``
+            The contents of this segment will be a copy of the given one. A typical use is
+            to make a copy of the ``.data`` section that is in RAM and needs to
+            be initialized (by the startup code) from a copy located in Flash memory::
+
+                programmable segment     .data_init      in:FLASH,mirror:.data
+        """
         if self.memory_map is None:
             raise MCUDefintitionError('SEGMENT outside memory map definition not allowed')
         segment_name = self.next_word()
@@ -235,7 +295,7 @@ class MCUDefintitions(rpn.RPN):
         else:
             # address range
             start, end = self.address_range(address)
-            self.memory_map[segment_name] = {'start':start, 'end':end}
+            self.memory_map[segment_name] = {'start': start, 'end': end}
         self.memory_map[segment_name]['order'] = self.order
         self.memory_map[segment_name]['flags'] = self.flags
         self.memory_map[segment_name]['__name__'] = segment_name
@@ -245,7 +305,21 @@ class MCUDefintitions(rpn.RPN):
 
     @rpn.word('SYMBOL')
     def word_SYMBOL(self, stack):
-        """Create a symbol."""
+        """\
+        Example::
+
+            symbol <name> <address>
+
+        Defines a symbol with the value specified. ``<address>`` can also be a computed
+        value. e.g. ``in:RAM,location:end``.
+
+        Supported are: ``in:<segment_name>`` and ``location:[start|end]``.  These
+        values are computed at load time, i.e. the segment still have the address
+        range specified in the definition (opposed to the values after the linker has
+        "shrinked" the segments to the size of actually present data). Note that
+        ``location:end`` is the segments last address plus one (end is exclusive in
+        this case).
+        """
         symbol_name = self.next_word()
         address = self.next_word()
         if ':' in address:
@@ -256,26 +330,39 @@ class MCUDefintitions(rpn.RPN):
                 self.memory_map[symbol_name][key] = value
         else:
             # address
-            self.memory_map[symbol_name] = {'address':int(address,16)}
+            self.memory_map[symbol_name] = {'address': int(address, 16)}
         self.memory_map[symbol_name]['__name__'] = symbol_name
         self.memory_map[symbol_name]['__type__'] = 'symbol'
 
     @rpn.word('NAME')
     def word_NAME(self, stack):
-        """Set name of current memory map definition."""
+        """\
+        Set the name of a memory map.
+        Example::
+
+            name <name>
+        """
         if self.memory_map is None:
             raise MCUDefintitionError('NAME outside memory map definition not allowed')
         self.memory_map['__name__'] = self.next_word()
 
     @rpn.word('BASED-ON')
     def word_BASED_ON(self, stack):
-        """Set name of base map for current memory map definition."""
+        """\
+        Tell that a memory map definition builds on an other definition.
+        All the definitions are merged when used.
+        Example::
+
+            based-on <name>
+
+        """
         if self.memory_map is None:
             raise MCUDefintitionError('BASED-ON outside memory map definition not allowed')
         self.memory_map['__based_on__'] = self.next_word()
 
 # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
+
 def parse_words(iterable):
     """\
     Parse a configuration file/text using the given iterable.
@@ -290,14 +377,15 @@ def expand_definition(memory_maps, name):
     Recursively expand the '__based_on__' keys to create a 'flat' definition
     for the given MCU name.
     """
-    map = dict(memory_maps[name]) # get a copy of the dict
+    map = dict(memory_maps[name])  # get a copy of the dict
     try:
         base = map.pop('__based_on__')
     except KeyError:
         pass
     else:
         map.update(expand_definition(memory_maps, base))
-    if '__name__' in map: del map['__name__']   # name was overwritten by lowest base
+    if '__name__' in map:
+        del map['__name__']   # name was overwritten by lowest base
     map['__name__'] = name
     return map
 
@@ -307,8 +395,9 @@ def load_internal():
     Load configuration file and only return a single, expanded memory map for
     given mcu_name.
     """
-    data = pkgutil.get_data('msp430.asm', 'definitions/msp430-mcu-list.txt')
-    return parse_words(rpn.words_in_string(data))
+    data = pkgutil.get_data('msp430.asm', 'definitions/msp430-mcu-list.txt').decode('utf-8')
+    return parse_words(rpn.words_in_string(data, name='msp430-mcu-list.txt'))
+
 
 def load_from_file(filename):
     """\
@@ -321,44 +410,47 @@ def load_from_file(filename):
 
 # test only
 if __name__ == '__main__':
-    from optparse import OptionParser
+    import argparse
+    import sys
     from pprint import pprint
 
-    parser = OptionParser()
+    parser = argparse.ArgumentParser()
 
-    parser.add_option("-l", "--list",
-            action = "store_true",
-            dest = "list",
-            default = False,
-            help = "list available MCU names")
+    parser.add_argument('MCUNAME', nargs='*')
 
-    parser.add_option("-d", "--dump",
-            action = "store_true",
-            dest = "dump",
-            default = False,
-            help = "dump all data instead of pretty printing")
+    parser.add_argument(
+        '-l', '--list',
+        action='store_true',
+        default=False,
+        help='list available MCU names')
 
-    (options, args) = parser.parse_args()
+    parser.add_argument(
+        '-d', '--dump',
+        action='store_true',
+        default=False,
+        help='dump all data instead of pretty printing')
+
+    args = parser.parse_args()
 
     try:
         memory_maps = load_internal()
-    except rpn.RPNError, e:
-        print "%s:%s: %s" % (e.filename, e.lineno, e)
+    except rpn.RPNError as e:
+        sys.stderr.write(u'{e.filename}:{e.lineno}: {e}\n'.format(e=e))
     else:
-        if options.list:
+        if args.list:
             for mcu in sorted(memory_maps):
-                print mcu
+                sys.stdout.write('{}\n'.format(mcu))
             #~ pprint(memory_maps)
-        for mcu in args:
-            print '== memory map for %s ==' % mcu
+        for mcu in args.MCUNAME:
+            print('== memory map for {} =='.format(mcu))
             memmap = expand_definition(memory_maps, mcu)
-            if options.dump:
+            if args.dump:
                 pprint(memmap)
             else:
                 for name, segment in sorted(memmap.items()):
                     if not name.startswith('__') and 'start' in segment:
-                        print '%-12s %08x-%08x %s' % (
-                                name,
-                                segment['start'],
-                                segment['end'],
-                                ','.join(segment['flags']))
+                        sys.stdout.write('{:12} {:08x}-{:08x} {}\n'.format(
+                            name,
+                            segment['start'],
+                            segment['end'],
+                            ','.join(segment['flags'])))
