@@ -1,10 +1,10 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2002-2010 Chris Liechti <cliechti@gmx.net>
-# All Rights Reserved.
-# Simplified BSD License (see LICENSE.txt for full text)
-
+# This file is part of https://github.com/zsquareplusc/python-msp430-tools
+# (C) 2002-2010 Chris Liechti <cliechti@gmx.net>
+#
+# SPDX-License-Identifier:    BSD-3-Clause
 """\
 This module defines a default target that represents a MSP430 device. It
 defines some common operations and the basics of a command line frontend using
@@ -39,7 +39,7 @@ F4x = "F4x family"
 
 # known device list
 DEVICEIDS = {
-#    CPUID   BSLVER family
+    # CPUID   BSLVER family
     (0x1132, None): F1x,      # F1122, F1132
     (0x1232, None): F1x,      # F1222, F1232
     (0xf112, None): F1x,      # F11x, F11x1, F11x1A
@@ -50,7 +50,8 @@ DEVICEIDS = {
     (0xf169, None): F1x,      # F16x
     (0xf16c, None): F1x,      # F161x
     (0xf227, None): F2x,      # F22xx
-    (0xf26f, None): F2x,
+    (0xf249, None): F2x,      # F22xx
+    (0xf26f, None): F2x,      # F24x
     (0xf413, None): F4x,
     (0xf427, None): F4x,      # FE42x, FW42x, F41(5,7), F42x0
     (0xf439, None): F4x,      # FG43x
@@ -58,6 +59,7 @@ DEVICEIDS = {
     (0xf46f, None): F4x,      # FG46xx
 }
 
+
 def identify_device(device_id, bsl_version):
     try:
         try:
@@ -65,15 +67,16 @@ def identify_device(device_id, bsl_version):
         except KeyError:
             return DEVICEIDS[device_id, None]
     except KeyError:
-        if device_id[0] == '0x1f': return F1x
-        if device_id[0] == '0x2f': return F2x
-        if device_id[0] == '0x4f': return F4x
+        if device_id >> 8 == 0xf1: return F1x
+        if device_id >> 8 == 0xf2: return F2x
+        if device_id >> 8 == 0xf4: return F4x
         raise KeyError('device type not known %04x/%04x' % (device_id, bsl_version))
 
 
 class UnsupportedMCUFamily(Exception):
     """This exception is raised when the CPU family is not compatible"""
 
+
 # i don't like how texts are re-wrapped and paragraphs are joined. get rid
 # of that "bug"
 class Formatter(IndentedHelpFormatter):
@@ -177,13 +180,12 @@ class Target(object):
         self.debug = False
         self.debug = True   # XXX
 
-
     def flash_segment_size(self, address):
         """Determine the Flash segment size"""
         # XXX make it device family aware
-        if address < 0x1000:
+        if address < 0x1100:
             modulo = 64
-        elif address < 0x1100:
+        elif address < 0x1200:
             modulo = 256
         else:
             modulo = 512
@@ -239,23 +241,31 @@ class Target(object):
         if self.verbose:
             sys.stderr.write('Upload by file: done\n')
 
-    def program_file(self, download_data=None):
-        """download data from self.download_data or the optional parameter"""
+    def program_file(self, download_data=None, quiet=False):
+        """\
+        download data from self.download_data or the optional parameter.
+        status messages on stderr are printed unless the quiet parameter is
+        true (this can e.g. used to download helper code)
+        """
+        if self.verbose and not quiet:
+            sys.stderr.write('Programming...\n')
         if download_data is None:
             download_data = self.download_data
         for segment in download_data:
-            if self.verbose > 1:
+            if self.verbose > 1 and not quiet:
                 sys.stderr.write("Write segment at 0x%04x %d bytes\n" % (segment.startaddress, len(segment.data)))
             data = segment.data
             # pad length if odd number of bytes
             if len(data) & 1:
                 data += '\xff'
             self.memory_write(segment.startaddress, data)
-        if self.verbose:
+        if self.verbose and not quiet:
             sys.stderr.write('Programming: OK\n')
 
     def verify_by_file(self):
         """upload and compare to self.download_data"""
+        if self.verbose:
+            sys.stderr.write('Verify by file...\n')
         for segment in self.download_data:
             if self.verbose > 1:
                 sys.stderr.write("Verify segment at 0x%04x %d bytes\n" % (segment.startaddress, len(segment.data)))
@@ -268,11 +278,13 @@ class Target(object):
 
     def erase_check_by_file(self):
         """upload address ranges used in self.download_data and check if memory erased (0xff)"""
+        if self.verbose:
+            sys.stderr.write('Erase check by file...\n')
         for segment in self.download_data:
             if self.verbose > 1:
                 sys.stderr.write("Erase check segment at 0x%04x %d bytes\n" % (segment.startaddress, len(segment.data)))
             data = self.memory_read(segment.startaddress, len(segment.data))
-            if data != '\xff'*len(segment.data):
+            if data != '\xff' * len(segment.data):
                 raise Exception("erase check failed at 0x%04x" % (segment.startaddress,))
             # XXX show hex DIFF
         if self.verbose:
@@ -282,6 +294,8 @@ class Target(object):
         """\
         Erase Flash segments that will be used by the data in self.download_data.
         """
+        if self.verbose:
+            sys.stderr.write('Erase by file...\n')
         for segment in self.download_data:
             address = segment.startaddress
             # mask address to get to segment start
@@ -295,7 +309,6 @@ class Target(object):
         if self.verbose:
             sys.stderr.write('Erase by file: OK\n')
 
-
     # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     # command line interface implementation
     # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -304,35 +317,40 @@ class Target(object):
         """create OptionParser with default options"""
         self.parser = OptionParser(usage="%prog [OPTIONS] [FILE [FILE...]]", formatter=Formatter())
 
-        self.parser.add_option("--debug",
-                help="print debug messages and tracebacks (development mode)",
-                dest="debug",
-                default=False,
-                action='store_true')
+        self.parser.add_option(
+            "--debug",
+            help="print debug messages and tracebacks (development mode)",
+            dest="debug",
+            default=False,
+            action='store_true')
 
-        self.parser.add_option("-v", "--verbose",
-                help="show more messages (can be given multiple times)",
-                dest="verbose",
-                default=1,
-                action='count')
+        self.parser.add_option(
+            "-v", "--verbose",
+            help="show more messages (can be given multiple times)",
+            dest="verbose",
+            default=1,
+            action='count')
 
-        self.parser.add_option("-q", "--quiet",
-                help="suppress all messages",
-                dest="verbose",
-                action='store_const',
-                const=0)
+        self.parser.add_option(
+            "-q", "--quiet",
+            help="suppress all messages",
+            dest="verbose",
+            action='store_const',
+            const=0)
 
-        self.parser.add_option("--time",
-                help="measure time",
-                dest="time",
-                action="store_true",
-                default=False)
+        self.parser.add_option(
+            "--time",
+            help="measure time",
+            dest="time",
+            action="store_true",
+            default=False)
 
-        self.parser.add_option("-S", "--progress",
-                dest="progress",
-                help="show progress while programming",
-                default=False,
-                action='store_true')
+        self.parser.add_option(
+            "-S", "--progress",
+            dest="progress",
+            help="show progress while programming",
+            default=False,
+            action='store_true')
 
         group = OptionGroup(self.parser, "Data input", """\
 File format is auto detected, unless --input-format is used.
@@ -343,15 +361,15 @@ Multiple files can be given on the command line, all are merged before the
 download starts. "-" reads from stdin.
     """)
 
-        group.add_option("-i", "--input-format",
-                dest="input_format",
-                help="input format name (%s)" % (', '.join(memory.load_formats),),
-                default=None,
-                metavar="TYPE")
+        group.add_option(
+            "-i", "--input-format",
+            dest="input_format",
+            help="input format name (%s)" % (', '.join(memory.load_formats),),
+            default=None,
+            metavar="TYPE")
 
         self.parser.add_option_group(group)
 
-
         group = OptionGroup(self.parser, "Flash erase", """\
 Multiple --erase options are allowed. It is also possible to use address
 ranges such as 0xf000-0xf0ff or 0xf000/4k.
@@ -359,36 +377,41 @@ ranges such as 0xf000-0xf0ff or 0xf000/4k.
 NOTE: SegmentA on F2xx is NOT erased with --mass-erase, that must be
 done separately with --erase=0x10c0 or --info-erase".
 """)
-        group.add_option("-e", "--mass-erase",
-                dest="do_mass_erase",
-                help="mass erase (clear all flash memory)",
-                default=False,
-                action='store_true')
+        group.add_option(
+            "-e", "--mass-erase",
+            dest="do_mass_erase",
+            help="mass erase (clear all flash memory)",
+            default=False,
+            action='store_true')
 
-        group.add_option("-m", "--main-erase",
-                dest="do_main_erase",
-                help="erase main flash memory only",
-                default=False,
-                action='store_true')
+        group.add_option(
+            "-m", "--main-erase",
+            dest="do_main_erase",
+            help="erase main flash memory only",
+            default=False,
+            action='store_true')
 
-        group.add_option("--info-erase",
-                dest="do_info_erase",
-                help="erase info flash memory only (0x1000-0x10ff)",
-                default=False,
-                action='store_true')
+        group.add_option(
+            "--info-erase",
+            dest="do_info_erase",
+            help="erase info flash memory only (0x1000-0x10ff)",
+            default=False,
+            action='store_true')
 
-        group.add_option("-b", "--erase-by-file",
-                dest="do_erase_by_file",
-                help="erase only Flash segments where new data is downloaded",
-                default=False,
-                action='store_true')
+        group.add_option(
+            "-b", "--erase-by-file",
+            dest="do_erase_by_file",
+            help="erase only Flash segments where new data is downloaded",
+            default=False,
+            action='store_true')
 
-        group.add_option("--erase",
-                dest="erase_list",
-                help="selectively erase segment at the specified address or address range",
-                default=[],
-                action='append',
-                metavar="ADDRESS")
+        group.add_option(
+            "--erase",
+            dest="erase_list",
+            help="selectively erase segment at the specified address or address range",
+            default=[],
+            action='append',
+            metavar="ADDRESS")
 
         self.parser.add_option_group(group)
 
@@ -404,33 +427,36 @@ only "-V" does a "check by file" of a programmed device without programming.
 
 Don't forget to erase ("-e", "-b" or "-m") before programming flash!
 """)
-        group.add_option("-E", "--erase-check",
-                dest="do_erase_check",
-                help="erase check by file",
-                default=False,
-                action='store_true')
+        group.add_option(
+            "-E", "--erase-check",
+            dest="do_erase_check",
+            help="erase check by file",
+            default=False,
+            action='store_true')
 
-        group.add_option("-P", "--program",
-                dest="do_program",
-                help="program file",
-                default=False,
-                action='store_true')
+        group.add_option(
+            "-P", "--program",
+            dest="do_program",
+            help="program file",
+            default=False,
+            action='store_true')
 
-        group.add_option("-V", "--verify",
-                dest="do_verify",
-                help="verify by file",
-                default=False,
-                action='store_true')
+        group.add_option(
+            "-V", "--verify",
+            dest="do_verify",
+            help="verify by file",
+            default=False,
+            action='store_true')
 
-        group.add_option("-U", "--upload-by-file",
-                dest="do_upload_by_file",
-                help="upload the memory that is present in the given file(s)",
-                default=False,
-                action='store_true')
+        group.add_option(
+            "-U", "--upload-by-file",
+            dest="do_upload_by_file",
+            help="upload the memory that is present in the given file(s)",
+            default=False,
+            action='store_true')
 
         self.parser.add_option_group(group)
 
-
         group = OptionGroup(self.parser, "Data upload", """\
 This can be used to read out the device memory.
 It is possible to use address ranges such as 0xf000-0xf0ff or 0xf000/256, 0xfc00/1k.
@@ -438,53 +464,60 @@ It is possible to use address ranges such as 0xf000-0xf0ff or 0xf000/256, 0xfc00
 Multiple --upload options are allowed.
 """)
 
-        group.add_option("-u", "--upload",
-                dest="upload_list",
-                metavar="ADDRESS",
-                help='upload a data block, can be passed multiple times',
-                default=[],
-                action='append')
+        group.add_option(
+            "-u", "--upload",
+            dest="upload_list",
+            metavar="ADDRESS",
+            help='upload a data block, can be passed multiple times',
+            default=[],
+            action='append')
 
-        group.add_option("-o", "--output",
-                dest="output",
-                help="write uploaded data to given file",
-                metavar="DESTINATION")
+        group.add_option(
+            "-o", "--output",
+            dest="output",
+            help="write uploaded data to given file",
+            metavar="DESTINATION")
 
-        group.add_option("-f", "--output-format",
-                dest="output_format",
-                help="output format name (%s), default:%%default" % (', '.join(memory.save_formats),),
-                default="hex",
-                metavar="TYPE")
+        group.add_option(
+            "-f", "--output-format",
+            dest="output_format",
+            help="output format name (%s), default:%%default" % (', '.join(memory.save_formats),),
+            default="hex",
+            metavar="TYPE")
 
         self.parser.add_option_group(group)
 
         group = OptionGroup(self.parser, "Do before exit")
 
-        group.add_option("-x", "--execute",
-                dest="do_run",
-                metavar="ADDRESS",
-                type="int",
-                help='start program execution at specified address, might only be useful in conjunction with --wait',
-                default=None,
-                action='store')
+        group.add_option(
+            "-x", "--execute",
+            dest="do_run",
+            metavar="ADDRESS",
+            type="int",
+            help='start program execution at specified address, might only be useful in conjunction with --wait',
+            default=None,
+            action='store')
 
-        group.add_option("-r", "--reset",
-                dest="do_reset",
-                help="perform a normal device reset that will start the program that is specified in the reset interrupt vector",
-                default=False,
-                action='store_true')
+        group.add_option(
+            "-r", "--reset",
+            dest="do_reset",
+            help="perform a normal device reset that will start the program that is specified in the reset interrupt vector",
+            default=False,
+            action='store_true')
 
-        group.add_option("-w", "--wait",
-                dest="do_wait",
-                help="wait for <ENTER> before closing the port",
-                default=False,
-                action='store_true')
+        group.add_option(
+            "-w", "--wait",
+            dest="do_wait",
+            help="wait for <ENTER> before closing the port",
+            default=False,
+            action='store_true')
 
-        group.add_option("--no-close",
-                dest="no_close",
-                help="do not close port on exit",
-                default=False,
-                action='store_true')
+        group.add_option(
+            "--no-close",
+            dest="no_close",
+            help="do not close port on exit",
+            default=False,
+            action='store_true')
 
         self.parser.add_option_group(group)
 
@@ -494,7 +527,6 @@ Multiple --upload options are allowed.
         """Store a function to be called and parameters in the list of actions"""
         self.action_list.append((function, args, kwargs))
 
-
     def remove_action(self, function):
         """Remove a function from the list of actions"""
         for entry in self.action_list:
@@ -507,12 +539,13 @@ Multiple --upload options are allowed.
     # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     def parse_args(self):
+        (self.options, self.args) = None, []  # parse_args may terminate...
         (self.options, self.args) = self.parser.parse_args()
 
         self.debug = self.options.debug
         self.verbose = self.options.verbose
 
-        if self.verbose > 3 :
+        if self.verbose > 3:
             level = logging.DEBUG
         elif self.verbose > 2:
             level = logging.INFO
@@ -562,7 +595,7 @@ Multiple --upload options are allowed.
                 adr, adr2 = parseAddressRange(a)
                 if adr2 is not None:
                     while adr <= adr2:
-                        if not (0x1000 <= adr <= 0xffff):
+                        if not (0x1000 <= adr <= 0xffffff):
                             self.parser.error("Start address for --erase is not within Flash memory: 0x%04x" % (adr,))
                         elif adr < 0x1100:
                             modulo = 64     # F2xx XXX: on F1xx/F4xx are segments erased twice
@@ -575,7 +608,7 @@ Multiple --upload options are allowed.
                         adr = adr + modulo
                 else:
                     self.add_action(self.erase, adr)
-            except ValueError, e:
+            except ValueError as e:
                 self.parser.error("--erase: %s" % e)
 
         default_action = True
@@ -592,7 +625,6 @@ Multiple --upload options are allowed.
             self.add_action(self.upload_by_file)
             default_action = False
 
-
         # as default action (no other given by user), program if a file is given
         if default_action and self.args:
             self.add_action(self.program_file)
@@ -603,7 +635,7 @@ Multiple --upload options are allowed.
                 if end is None:
                     end = start + 15
                 self.add_action(self.upload, start, end)
-            except ValueError, e:
+            except ValueError as e:
                 self.parser.error("--upload: %s" % e)
 
         if self.options.do_reset:
@@ -642,7 +674,6 @@ Multiple --upload options are allowed.
                         format=self.options.input_format)
             self.download_data.merge(data)
 
-
     def do_the_work(self):
         """\
         Do the actual work, such as upload and download.
@@ -661,56 +692,59 @@ Multiple --upload options are allowed.
                 sys.stderr.write("   <no actions>\n")
             sys.stderr.flush()
 
-        abort_due_to_error = True
         self.open_connection()
-        try:
-            # work through action list
-            for f, args, kwargs in self.action_list:
-                f(*args, **kwargs)
+        # work through action list
+        for f, args, kwargs in self.action_list:
+            f(*args, **kwargs)
 
-            # output uploaded data
-            if self.upload_data is not None:
-                memory.save(self.upload_data, self.output, self.options.output_format)
+        # output uploaded data
+        if self.upload_data is not None:
+            memory.save(self.upload_data, self.output, self.options.output_format)
 
-            if self.options.do_wait:                        # wait at the end if desired
-                if self.verbose:
-                    sys.stderr.write("Press <ENTER> ...\n") # display a prompt
-                    sys.stderr.flush()
-                raw_input()                                 # wait for newline
-
-            abort_due_to_error = False
-        finally:
-            if abort_due_to_error:
-                sys.stderr.write("Cleaning up after error...\n")
-            if not self.options.no_close:
-                self.close_connection()                     # release communication port
-            elif self.verbose:
-                sys.stderr.write("WARNING: port is left open (--no-close)\n")
+        if self.options.do_wait:                         # wait at the end if desired
+            if self.verbose:
+                sys.stderr.write("Press <ENTER> ...\n")  # display a prompt
+                sys.stderr.flush()
+            input()                                      # wait for newline
 
     # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     def main(self):
         """Main command line entry"""
         start_time = None
+        abort_due_to_error = False
         try:
             self.create_option_parser()
             self.add_extra_options()
             self.parse_args()
+            abort_due_to_error = True
             self.parse_extra_options()
             if self.options.time:
                 start_time = time.time()
             self.do_the_work()
+            abort_due_to_error = False
         except SystemExit:
+            abort_due_to_error = False                      # lets assume they are not becuase of an internal error
             raise                                           # let pass exit() calls
         except KeyboardInterrupt:
             if self.debug: raise                            # show full trace in debug mode
             sys.stderr.write("\nAbort on user request.\n")  # short message in user mode
             sys.exit(1)                                     # set error level for script usage
-        except Exception, msg:                              # every Exception is caught and displayed
+        except Exception as msg:                            # every Exception is caught and displayed
             if self.debug: raise                            # show full trace in debug mode
-            sys.stderr.write("\nAn error occurred:\n%s\n" % msg) # short message in user mode
+            sys.stderr.write("\nAn error occurred:\n%s\n" % msg)  # short message in user mode
             sys.exit(1)                                     # set error level for script usage
         finally:
+            if abort_due_to_error:
+                sys.stderr.write("Cleaning up after error...\n")
+            if self.options is not None and not self.options.no_close:
+                try:
+                    self.close_connection()                     # release communication port
+                except Exception as msg:                        # every Exception is caught and displayed
+                    if self.debug: raise                        # show full trace in debug mode
+                    sys.stderr.write("\nAn error occurred during shutdown:\n%s\n" % msg)  # short message in user mode
+            elif self.verbose:
+                sys.stderr.write("WARNING: port is left open (--no-close)\n")
             if start_time is not None:
                 end_time = time.time()
                 sys.stderr.write("Time: %.1f s\n" % (end_time - start_time))
@@ -720,4 +754,3 @@ Multiple --upload options are allowed.
 if __name__ == '__main__':
     t = Target()
     t.main()
-
